/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
BookmarkTools.include("external/webtoolkit.utf8.js");
BookmarkTools.include("external/webtoolkit.base64.js");

BookmarkTools.namespace("utils");
BookmarkTools.namespace("dataTypes");

/**
	@class Utility class to receive the data type of a javascript variable. <br />
*/
BookmarkTools.utils.ObjectUtils = {

	/**
		Checks if the passed parameter is undefined.
		@param {object} object
		@return true if undefined, otherwise false
	*/
	isUndefined : function(object){
		return typeof object === "undefined";
	},
	
	/**
		Checks if the passed parameter is not undefined.
		@param {object} object
		@return true if not undefined, otherwise false
	*/
	isNotUndefined : function(object){
		return !this.isUndefined(object);
	},
	
	/**
		Checks if the passed parameter is a function.
		@param {object} object
		@return true if function, otherwise false
	*/
	isFunction : function(object){
		return typeof object === "function";
	},
	
	/**
		Checks if the passed parameter is an array.
		@param {object} object
		@return true if array, otherwise false
	*/
	isArray : function(object){
		if(this.isUndefined(object) || object == null)
			return false;

		var constr = object.constructor.toString();
		var map = BookmarkTools.dataTypes.ExtendedHash.toString();
	
		return constr.indexOf("Array") != -1 && constr != map;
	},
	
	/**
		Checks if the passed parameter is a number.
		@param {object} object
		@return true if number, otherwise false
	*/
	isNumber : function(object){
		if(this.isUndefined(object) || object == null)
			return false;

		return object.constructor.toString().indexOf("Number") != -1;
	},
	
	/**
		Checks if the passed parameter is a string.
		@param {object} object
		@return true if string, otherwise false
	*/
	isString : function(object){
		if(this.isUndefined(object) || object == null)
			return false;

		return object.constructor.toString().indexOf("String") != -1;
	},
	
	/**
		Checks if the passed parameter is an instance of nsIRDFResource.
		@param {object} object
		@return true if nsIRDFResource, otherwise false
	*/
	isRDFResource : function(object){
		if(this.isUndefined(object) || object == null)
			return false;
			
		return (object instanceof Ci.nsIRDFResource);
	}
	
};

/**
	@class Utility class that should help to work with strings. <br />
	@requires BookmarkTools.utils.ObjectUtils
*/
BookmarkTools.utils.StringUtils = {
	
	/** CONSTANT: empty string */
	EMPTY : '',
	
	/** CONSTANT: line feed */
	LF : '\n',
	
	/** CONSTANT: space */
	SPACE : ' ',
	
	/** CONSTANT: tab */
	TAB : '\t',
	
	/**
		Checks if a string is empty or not (ignores whitespaces).
		@param {string} str some string value
		@return true if empty, otherwise false
	*/
	isEmpty : function(str){
		if(!BookmarkTools.utils.ObjectUtils.isString(str))
			throw 'ERROR: Invalid parameter: ' + str;
			
		return /^\s*$/.test(str);
	},
	
	/**
		Checks if a string is not empty (ignores whitespaces).
		@param {string} str some string value
		@return true if not empty, otherwise false
	*/
	isNotEmpty : function(str){
		return !this.isEmpty(str);
	},
	
	/**
		Truncates a string and append '...' at the end.
		@param {string} str some string value
		@param {number} length number of shown charaters after truncation
		@param {string} truncation some truncation string (e.g. '[..]' or '...')
		@return truncated string
	*/
	truncate : function(str, length, truncation) {
		if(!BookmarkTools.utils.ObjectUtils.isString(str))
			throw 'ERROR: Invalid parameter: ' + str;
			
		length = length || 30;
		truncation = BookmarkTools.utils.ObjectUtils.isUndefined(truncation) ? '...' : truncation;
		return str.length > length ? str.slice(0, length - truncation.length) + truncation : str;
	},
	
	/**
		Fast trim implementation to handle long strings as fast as short strings.
		@param {string} str some string value
		@return trimmed string
	*/
	trim : function(str){
		if(!BookmarkTools.utils.ObjectUtils.isString(str))
			throw 'ERROR: Invalid parameter: ' + str;
			
		var start = -1, end = str.length;  
		while(str.charCodeAt(--end) < 33);  
		while(str.charCodeAt(++start) < 33);  
		return str.slice(start, end + 1); 
	},
	
	/**
		Utility function to set a character on any position in the given string.
		@param {string} str some string value
		@param {number} i position to set the character
		@param {string} c character
		@return string with character c on position i
	*/
	setCharAt : function(str, i, c){
		if(!BookmarkTools.utils.ObjectUtils.isString(str))
			throw 'ERROR: Invalid parameter: ' + str;
			
		if(i > str.length - 1 || i < 0) return str;
		return str.substr(0, i) + c + str.substr(i + 1);
	},

	/**
		Compares two strings and ignores case.
		@param {string} a some string value
		@param {string} b some string value
		@return true if a equals b, otherwise false
	*/
	equalsIgnoreCase : function(a, b){
		if(!BookmarkTools.utils.ObjectUtils.isString(a) || !BookmarkTools.utils.ObjectUtils.isString(b))
			throw 'ERROR: Invalid parameter: ' + a + ", " + b;
			
		var astr = a.toLowerCase();
		var bstr = b.toLowerCase();
		return astr == bstr;
	},

	/**
		Checks if string a contains string b.
		@param {string} a some string value
		@param {string} b some string value
		@return true if a contains b, otherwise false
	*/
	contains : function(a, b){
		if(!BookmarkTools.utils.ObjectUtils.isString(a) || !BookmarkTools.utils.ObjectUtils.isString(b))
			throw 'ERROR: Invalid parameter: ' + a + ", " + b;
			
		return a.indexOf(b) != -1;
	},

	/**
		Converts the passed parameter to an instance of type nsiURI.
		@param {string} str some string value
		@return an instance of type nsiURI
	*/
	nsiURI : function(str){
		if(!BookmarkTools.utils.ObjectUtils.isString(str))
			throw 'ERROR: Invalid parameter: ' + str;
			
		var ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
		return ioService.newURI(str, null, null);
	},
		
	/**
		Returns an bytes array of the gives string.
		@param {string} str some string value
		@return bytes array (with unsigned bytes)
	*/
	bytesOf : function(str){
		if(!BookmarkTools.utils.ObjectUtils.isString(str))
			throw 'ERROR: Invalid parameter: ' + str;
	
		var ch, st, re = [];
		for (var i = 0, l = str.length ; i < l; i++) {
			ch = str.charCodeAt(i);  // get char 
			st = [];                 // set up "stack"
			do {
				st.push(ch & 0xFF);  // push byte to stack
				ch = ch >> 8;        // shift value down by 1 byte
			} while (ch);
			// add stack contents to result
			// done because chars have "wrong" endianness
			re = re.concat(st.reverse());
		}
		// return an array of bytes
		return re;
	},
	
	/**
		Converts a given byte array back to a string value.
		@param {array} array byte array
		@return string value
	*/
	stringOf : function(array){
		if(!BookmarkTools.utils.ObjectUtils.isArray(array))
			throw 'ERROR: Invalid parameter: ' + array;
	
		var res = this.EMPTY;
		for(var i = 0, l = array.length ;i < l; i++){
			res += String.fromCharCode(array[i]);
		}
		return res;
	},
	
	/**
		Encodes a string with utf-8.
		@param {string} str some string value
		@return utf-8 encoded string
	*/
	encodeUTF8 : function(str){
		if(!BookmarkTools.utils.ObjectUtils.isString(str))
			throw 'ERROR: Invalid parameter: ' + str;

		return Utf8.encode(str);
	},
	
	/**
		Decodes an utf-8 encoded string.
		@param {string} str some utf-8 encoded string value
		@return some string value
	*/
	decodeUTF8 : function(str){
		if(!BookmarkTools.utils.ObjectUtils.isString(str))
			throw 'ERROR: Invalid parameter: ' + str;

		return Utf8.decode(str);
	},
	
	/**
		Encodes an byte array to base64.
		@param {array} array byte array of string value
		@return base64 encoded string
	*/
	encodeBase64 : function(array){
		return Base64.encode(array, true);
	},
	
	/**
		Decodes an base64 encoded string.
		@param {string} str base64 string value
		@return decoded string
	*/
	decodeBase64 : function(str){
		if(!BookmarkTools.utils.ObjectUtils.isString(str))
			throw 'ERROR: Invalid parameter: ' + str;

		return Base64.decode(str);
	}

};

/**
	@class Utility class to handle xml documents. <br />
	@requires BookmarkTools.utils.StringUtils
*/
BookmarkTools.utils.XMLUtils = {
	
	/**
		Return an empty xml document with the specified root and doctype.
		@param {string} publicID
		@param {string} systemID
		@param {string} qualifiedName xml root name
		@return an empty xml document
	*/
	getEmptyXML : function(publicID, systemID, qualifiedName){
		var domDocumentType = document.implementation.createDocumentType(
			qualifiedName, publicID, systemID
		);
    	return document.implementation.createDocument(
			BookmarkTools.utils.StringUtils.EMPTY, qualifiedName, domDocumentType
		);
	},
	
	/**
		Finalize the given xml document, appends initial line and serialize.
		@param {dom document} xmlDocument
		@return xml as string
	*/
	toString : function(xmlDocument){
		var header = '<?xml version="1.0" encoding="UTF-8"?>\n';
		var serializer = new XMLSerializer();

		return BookmarkTools.utils.StringUtils.encodeUTF8(
			header + serializer.serializeToString(xmlDocument)
		);
	},
	
	/**
		Parser for xml strings. Returns parsed document or null in case of errors.
		@param {string} xmlString xml as string
		@return dom document or null
	*/
	parseXML : function(xmlString){
		var parser = new DOMParser();
		var doc = parser.parseFromString(xmlString, "text/xml");
		
		if(doc.documentElement.nodeName != "parsererror")
			return doc;
		return null;
	}
	
};

/**
	@class Utility class to handle javascript numbers. <br />
	@requires BookmarkTools.utils.ObjectUtils
*/
BookmarkTools.utils.NumberUtils = {

	/**
		Fast parseInt implementation.
		@param {string} str string to parse
		@return parsed number
	*/
	parseInt : function(str){
		return ~~(1 * str);
	},
	
	/**
		Fast parseFloat implementation.
		@param {string} str string to parse
		@return parsed number
	*/
	parseFloat : function(str){
		return (1 * str);
	},

	/**
		Checks if a number is positive (includes 0).
		@param {number} number number to check
		@return true if positive, otherwise false
	*/
	isPositive : function(number){
		if(!BookmarkTools.utils.ObjectUtils.isNumber(number))
			throw 'ERROR: Invalid parameter: ' + number;
			
		return number >= 0;
	},

	/**
		Checks if a number is negative (includes 0).
		@param {number} number number to check
		@return true if negative, otherwise false
	*/
	isNegative : function(number){
		if(!BookmarkTools.utils.ObjectUtils.isNumber(number))
			throw 'ERROR: Invalid parameter: ' + number;
			
		return number <= 0;
	},

	/**
		Checks if a number is zero.
		@param {number} number number to check
		@return true if zero, otherwise false
	*/
	isZero : function(number){
		if(!BookmarkTools.utils.ObjectUtils.isNumber(number))
			throw 'ERROR: Invalid parameter: ' + number;
			
		return number == 0;
	}
	
};

/**
	@class Utility class to handle arrays. <br />
	@requires BookmarkTools.utils.ObjectUtils
*/
BookmarkTools.utils.ArrayUtils = {

	/**
		Checks if an array is empty.
		@param {array} a array to check
		@return true if empty, otherwise false
	*/
	isEmpty : function(a){
		if(!BookmarkTools.utils.ObjectUtils.isArray(a))
			throw 'ERROR: Invalid parameter: ' + a;
	
		return a.length == 0;
	},
	
	/**
		Checks if an array is not empty.
		@param {array} a array to check
		@return true if not empty, otherwise false
	*/
	isNotEmpty : function(a){
		if(!BookmarkTools.utils.ObjectUtils.isArray(a))
			throw 'ERROR: Invalid parameter: ' + a;
			
		return a.length != 0;
	},
	
	/**
		Converts an array to string.
		@param {array} a array to convert
		@return string value (e.g. ["a", "b", "c"] or [1,2,3])
	*/
	toString : function(a){
		if(!BookmarkTools.utils.ObjectUtils.isArray(a))
			throw 'ERROR: Invalid parameter: ' + a;
		return "[" + a.join(", ") + "]";
	},
	
	/**
		Finds an element using binary search (runtime O(logn)).
		@param {array} a
		@param {object} b
		@return array index if a contains b, otherwise -1
	*/
	search : function(a, b){
		if(!BookmarkTools.utils.ObjectUtils.isArray(a))
			throw 'ERROR: Invalid parameter: ' + a;
			
		var left = 0;
		var right = a.length - 1;
		var mid;
		
		while (left <= right){
			mid = Math.floor((left + right)/2);
			if (a[mid] == b)
				return mid;
			else if (a[mid] < b)
				left = mid + 1;
			else
				right = mid - 1;
		}
		return -1;
	},
	
	/**
		Merges a and b to an unique array in which
		each element occurs only one time (runtime O(nlogn + mlogm)).
		@param {array} a (|a| = n)
		@param {array} b (|b| = m)
		@return unique array
	*/
	merge : function(a, b){
		if(!BookmarkTools.utils.ObjectUtils.isArray(a))
			throw 'ERROR: Invalid parameter: ' + a;
			
		if(!BookmarkTools.utils.ObjectUtils.isArray(b))
			throw 'ERROR: Invalid parameter: ' + b;	
			
		if(this.isEmpty(a) && this.isEmpty(b)){
			return [];
		}
		
		var merged = [], i, l, tmp, index;
		for(i = 0, l = a.length ;i < l; i++){
			tmp = a[i];
			index = this.search(merged, tmp);
			if(index == -1)
				merged.push(tmp);
		}
		
		for(i = 0, l = b.length ;i < l; i++){
			tmp = b[i];
			index = this.search(merged, tmp);
			if(index == -1)
				merged.push(tmp);
		}
		return merged;
	}

};

/** 
	@class Utility class to handle json strings which are support by gecko from version 1.9.1, used by firefox 3.5 and seamonkey 2.0.  <br />
*/
BookmarkTools.utils.JSONUtils = {
	/**
		Converts javascript objects into a json strings.
		@param {object} object object to convert
		@return json string
	*/
	toJSON : function(object){
		return JSON.stringify(object);
	},
	
	/**
		Opposite direction, converts json strings back to js objects.
		@param {string} str str to convert
		@return js object
	*/
	fromJSON : function(str){
		return JSON.parse(str);
	}
};

/** 
	@class Helper class to create xpcom objects with a single function call. <br />
*/
BookmarkTools.utils.NativeComponents = {
	
	/**
		Returns an instance of nsIMutableArray.
		@see <a href="https://developer.mozilla.org/en/NsIMutableArray">nsIMutableArray</a>
		@return instance of nsIMutableArray
	*/
	nsIMutableArray : function(){
		return Cc["@mozilla.org/array;1"].createInstance(Ci.nsIMutableArray);
	},

	/**
		Returns an array of nsIXULWindow objects.
		@see <a href="https://developer.mozilla.org/en/nsIWindowMediator">nsIWindowMediator</a>, 
		<a href="http://www.oxymoronical.com/experiments/apidocs/interface/nsIXULWindow">nsIXULWindow</a>
		@return array of nsIXULWindow objects
	*/
	nsIWindowMediator : function(type){
		var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
		
		var w = [];
		var e = wm.getEnumerator(type);
		
		while(e.hasMoreElements())
			w.push(e.getNext());
		return w;
	},

	/**
		Returns an object of type nsIDOMWindowInternal  that represents the current browser window.
		@see <a href="https://developer.mozilla.org/en/NsIDOMWindowInternal">nsIDOMWindowInternal </a>
		@return instance of nsIDOMWindowInternal
	*/
	nsIBrowserWindow : function(){
		var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
		var mainWindow = wm.getMostRecentWindow("navigator:browser");
		//var browser = mainWindow.getBrowser();
		return mainWindow;
	}
	
};

/** 
	@class Data type that extends an assoziative array.  <br />
*/
BookmarkTools.dataTypes.ExtendedHash = function(){
	/*
		Note:
		Using [] instead of {} has the following effects.
		On the one hand it is faster accessing an element. On
		the other hand it is possible that elements overwrites
		native array properties (e.g. _hash["join"] = 0 overwrites
		the native join function). In this implementation
		native functions of _hash aren't required and so i choosed
		[] for fast element handling.
	*/
	var _hash = [];
	var _trash = [];
	var _length = 0;
	
	/**
		Returns all keys of the map (runtime O(n) because keys are not saved separatly).
		@return an array of the maps keys
	*/
	this.keys = function(){
		var _keys = [], index = 0, i;
		for(i in _hash)
			_keys[index++] = i;
		return _keys;
	};
	
	/**
		Puts a new key/value pair into the map. Collisions are stored in
		a separate list called trash.
		@param {object} key identifier
		@param {object} value value to store
	*/
	this.put = function(key, value){
		if(_hash[key]){
			_trash.push(value);
		}else {
			_hash[key] = value;
			_length++;
		}
	};
	
	/**
		Returns the value of the given key. If the map doesn't contains
		the key it returns undefined.
		@param {object} key identifier
		@return value or undefined
	*/
	this.get = function(key){
		return _hash[key];
	};
	
	/**
		Removes an key/value entry permanently from the map. If the map doesn't contains
		the key it returns undefined.
		@param {object} key identifier
		@return value or undefined
	*/
	this.remove = function(key){
		var elem;
		if(this.contains(key)){
			elem = _hash[key];
			delete _hash[key];
			_length--;
		}
		return elem;
	};
	
	/**
		Checks if the map contains the given key.
		@param {object} key identifier
		@return true or false
	*/
	this.contains = function(key){
		return _hash[key] != undefined;
	};
	
	/**
		Returns the current entry count.
		@return number of entries
	*/
	this.length = function(){
		return _length;
	};
	
	/**
		Returns if the map is empty.
		@return true if empty, otherwise false
	*/
	this.isEmpty = function(){
		return _length == 0;
	};
	
	/**
		Removes and returns the head of the map.
		@return head of the map
	*/
	this.shift = function(){
		var i, val;
		for(i in _hash){
			if(_length != 0){
				val = _hash[i];
				delete _hash[i];
				_length--;
			}
			return val;
		}
	};
	
	/**
		Iterator thats calls the passed function with key/value pairs from the map.
		@param {function} f function that is called in each iteration with key and value
	*/
	this.each = function(f){
		if(_length != 0){
			var i;
			for(i in _hash){
				f(i, _hash[i]);
			}
		}
	};
	
	/**
		Returns the trash list. That means this list contains each 
		entry that collides with a existing one.
		@return {array} trash list
	*/
	this.trash = function(){
		return _trash;
	};
}

/**
	@class Common wrapper of the nsIPrefService.  <br />
*/
BookmarkTools.utils.Prefs = {

	/** Root node of the preference branch. */
	root : 'bookmarkTools',

	/**
		Returns an reference to the nsIPrefBranch factory.
		@return nsIPrefBranch
	*/
	instance : function(){
		return Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
	},
	
	/**
		Returns an reference to the nsIPrefService factory.
		@return nsIPrefService
	*/
	service : function(){
		return Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);
	},
	
	/**
		Returns an preference entry of type nsILocalFile.
		@param {string} key preference key
		@return returns the nsILocalFile value specified under the given key
	*/
	getFile : function(key){
		var prefs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).getBranch("bookmarkTools.");
		return prefs.getComplexValue(key, Ci.nsILocalFile);
	},
	
	/**
		Returns an preference entry of type boolean.
		@param {string} key preference key
		@return returns the boolean value specified under the given key
	*/
	getBool : function(key){
		var inst = BookmarkTools.utils.Prefs.instance();
		return inst.getBoolPref(BookmarkTools.utils.Prefs.root + '.' + key);
	},
	
	/**
		Returns an preference entry of type string.
		@param {string} key preference key
		@return returns the string value specified under the given key
	*/
	getChar : function(key){
		var inst = BookmarkTools.utils.Prefs.instance();
		return inst.getCharPref(BookmarkTools.utils.Prefs.root + '.' + key);
	},
	
	/**
		Returns an preference entry of type number.
		@param {string} key preference key
		@return returns the number value specified under the given key
	*/
	getInt : function(key){
		var inst = BookmarkTools.utils.Prefs.instance();
		return inst.getIntPref(BookmarkTools.utils.Prefs.root + '.' + key);
	},
	
	/**
		Sets an nsILocalFile entry under the given key.
		@param {string} key preference key
		@param {nsILocalFile} file file instance
	*/
	setFile : function(key, file){
		//TODO duplicated code
		var prefs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).getBranch("bookmarkTools.");
		prefs.setComplexValue(key, Ci.nsILocalFile, file);
	},

	/**
		Sets an boolean entry under the given key.
		@param {string} key preference key
		@param {boolean} val boolean
	*/
	setBool : function(key, val){
		var inst = BookmarkTools.utils.Prefs.instance();
		inst.setBoolPref(BookmarkTools.utils.Prefs.root + '.' + key, val);
	},
	
	/**
		Sets an string entry under the given key.
		@param {string} key preference key
		@param {string} val string value
	*/
	setChar : function(key, val){
		var inst = BookmarkTools.utils.Prefs.instance();
		inst.setCharPref(BookmarkTools.utils.Prefs.root + '.' + key, val);
	},
	
	/**
		Sets an number entry under the given key.
		@param {string} key preference key
		@param {number} val number
	*/
	setInt : function(key, val){
		var inst = BookmarkTools.utils.Prefs.instance();
		inst.setIntPref(BookmarkTools.utils.Prefs.root + '.' + key, val);
	},
	
	/**
		Removes preference key permanently.
		@param {string} key preference key
	*/
	remove : function(key){
		var inst = BookmarkTools.utils.Prefs.instance();
		inst.clearUserPref(BookmarkTools.utils.Prefs.root + '.' + key);
	},
	
	/**
		Removes the first preference key that matches on the given key and uri.
		@param {string} key preference key
		@param {string} uri uri string
		@return true if removing was successfully, otherwise false
	*/
	removeAt : function(key, uri){
		var branch = BookmarkTools.utils.Prefs.getBranch(key);
		for(var i = 0, l = branch.length ;i < l; i++){ //TODO: performance
			var number = branch[i];
			var str = BookmarkTools.utils.Prefs.getChar(key + number);
			var json = BookmarkTools.utils.JSONUtils.fromJSON(str);
			
			if(json.uri == uri){
				BookmarkTools.utils.Prefs.remove(key + number);
				return true;
			}
		}
		return false;
	},
	
	clear : function(){
		var inst = BookmarkTools.utils.Prefs.instance();
		var l = this.length;
		
		for(var i = 0 ;i < l; i++)
			inst.clearUserPref(BookmarkTools.utils.Prefs.root + '.' + this.key + this.pop());
	},
	
	/**
		Removes the whole preference tree.
	*/
	clearAll : function(){
		BookmarkTools.utils.Prefs.getBranch(BookmarkTools.utils.StringUtils.EMPTY).clear();
	},
	
	/**
		Return an wrapped branch object (including toHash and clear function).
		@param {string} key preference key
		@return wrapped branch object
	*/
	getBranch : function(key){
		var branch = BookmarkTools.utils.Prefs.service().getBranch(BookmarkTools.utils.Prefs.root + '.' + key).getChildList(BookmarkTools.utils.StringUtils.EMPTY, {});
		branch.key = key;
		branch.toHash = BookmarkTools.utils.Prefs.toHash;
		branch.clear = BookmarkTools.utils.Prefs.clear;
		return branch;
	},
	
	/**
		Converts an wrapped branch object as ExtendedHash.
		@return BookmarkTools.dataTypes.ExtendedHash
	*/
	toHash : function(){
		var inst = BookmarkTools.utils.Prefs.instance();
		var hash = new BookmarkTools.dataTypes.ExtendedHash(), k = this.key;
		for(var i = 0, l = this.length ;i < l; i++){
			var elem = this[i];
			var _key = BookmarkTools.utils.Prefs.root + '.' + k + elem;
			var _value = BookmarkTools.utils.JSONUtils.fromJSON(inst.getCharPref(_key));
			hash.put(k + elem, _value);
		}
		return hash;
	}

};

/**
	@class Utility class for file handling. <br />
*/
BookmarkTools.utils.FileIO = {

	/**
		Returns the specified file. If it doesn't exists an empty file is created.
		@param {nsIFile} dir
		@param {string} filename
		@return Existing or new nsIFile instance
	*/
	generateFile : function(dir, filename){
		var file = dir.clone();
		file.append(filename);
		if(!file.exists())
			file.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, 0666);
		return file;
	},
	
	/**
		Returns an file object from preferences.
		@return nsIFile instance
	*/
	getFileObject : function(){
		var file = null;
		if(BookmarkTools.utils.Prefs.getBool("indexPathDefault")){
			file = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
		}else {
			file = BookmarkTools.utils.Prefs.getFile("indexPath");
		}
		return file;
	},
	
	/**
		Returns the build directory.
		@return build directory
	*/
	getBuildDir : function(){
		var file = this.getFileObject();

		file.append("cl_index");
		file.append("build");
		
		if(!file.exists() || !file.isDirectory()) 
			file.create(Ci.nsIFile.DIRECTORY_TYPE, 0777);

		return file.clone();
	},
	
	/**
		Returns the search directory.
		@return search directory
	*/
	getSearchDir : function(){
		var file = this.getFileObject();

		file.append("cl_index");
		file.append("search");
		
		if(!file.exists() || !file.isDirectory()) 
			file.create(Ci.nsIFile.DIRECTORY_TYPE, 0777);

		return file.clone();
	},
	
	/**
		Copies all files from bdir to sdir.
		@param {nsIFile} bdir
		@param {nsIFile} sdir
	*/
	move : function(bdir, sdir){
		var entries = bdir.directoryEntries;
		while(entries.hasMoreElements()){
			var file = entries.getNext().QueryInterface(Ci.nsIFile);
			if(!file.isDirectory()){
				// move file without renaming
				file.moveTo(sdir, file.leafName);
			}
		}
	},
	
	/**
		Clears search and build directory.
	*/
	clearDirectories : function(){
		this.clear(this.getSearchDir());
		this.clear(this.getBuildDir());
	},
	
	/**
		Clears log directory.
	*/
	clearLogs : function(){
		var dir = this.getFileObject();
		dir.append("cl_index");
		dir.append("log");
		this.clear(dir);
	},
	
	/**
		Clears the given directory.
		@param {nsIFile} dir
	*/
	clear : function(dir){
		if(dir.exists() && dir.isDirectory()){
			var entries = dir.directoryEntries;
			while(entries.hasMoreElements()){
				var file = entries.getNext().QueryInterface(Ci.nsIFile);
				if(file.isDirectory()){
					file.remove(true);
				}else {
					file.remove(false);
				}
			}
		}
	},
	
	/**
		Returns the size (in bytes) of an directory.
		@param {nsIFile} dir
	*/
	getDirectoryInfo : function(dir){
		var size = 0;
		var entries = dir.directoryEntries;
		
		while(entries.hasMoreElements()){
			var file = entries.getNext().QueryInterface(Ci.nsIFile);
			if(file.isDirectory()){
				size += BookmarkTools.utils.FileIO.getDirectoryInfo(file)
			}else {
				size += file.fileSize;
			}
		}
		return size;
	},
	
	/**
		Pretty print of bytes.
		@param {number} filesize number of bytes
	*/
	formatBytes : function(filesize){
		if (filesize >= 1073741824) {
			filesize = this.number_format(filesize / 1073741824, 2, '.', '') + ' Gb';
		} else {
			if (filesize >= 1048576) {
				filesize = this.number_format(filesize / 1048576, 2, '.', '') + ' Mb';
			} else {
				if (filesize >= 1024) {
					filesize = this.number_format(filesize / 1024, 0) + ' Kb';
				} else {
					filesize = this.number_format(filesize, 0) + ' bytes';
				}
			}
		}
		return filesize;
	},
	
	number_format : function(number, decimals, dec_point, thousands_sep){
		// http://kevin.vanzonneveld.net
		// +   original by: Jonas Raoni Soares Silva (http://www.jsfromhell.com)
		// +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
		// +     bugfix by: Michael White (http://crestidg.com)
		// +     bugfix by: Benjamin Lupton
		// +     bugfix by: Allan Jensen (http://www.winternet.no)
		// +    revised by: Jonas Raoni Soares Silva (http://www.jsfromhell.com)    
		// *     example 1: number_format(1234.5678, 2, '.', '');
		// *     returns 1: 1234.57     
	 
		var n = number, c = isNaN(decimals = Math.abs(decimals)) ? 2 : decimals;
		var d = dec_point == undefined ? "," : dec_point;
		var t = thousands_sep == undefined ? "." : thousands_sep, s = n < 0 ? "-" : "";
		var i = BookmarkTools.utils.NumberUtils.parseInt(n = Math.abs(+n || 0).toFixed(c)) + "", j = (j = i.length) > 3 ? j % 3 : 0;
		
		return s + (j ? i.substr(0, j) + t : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + t) + (c ? d + Math.abs(n - i).toFixed(c).slice(2) : "");
	},
	
	/**
		Writes an file line by line.
		@param {nsIFile} file
		@param {string} value
	*/
	writeLineToFile : function(file, value){
		if(file){
			var foStream = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);

			// use 0x02 | 0x10 to open file for appending.
			foStream.init(file, 0x02 | 0x08 | 0x10, 0666, 0); // write, create, truncate

			foStream.write(value, value.length);
			foStream.close();
		}
	},
	
	/**
		Reads the file line by line and returns it as BookmarkTools.dataTypes.ExtendedHash. 
		If no delimitter was specified each line occurs as key and the current line number 
		as value, otherwise the splitted pair defines key and value.
		@param {nsIFile} file
		@param {string} delim
		@return BookmarkTools.dataTypes.ExtendedHash
	*/
	readFileLineByLine : function(file, delim){
		// open an input stream from file
		var istream = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(Ci.nsIFileInputStream);
		istream.init(file, 0x01, 0444, 0);
		istream.QueryInterface(Ci.nsILineInputStream);

		// read lines into array
		var line = {}, lines = new BookmarkTools.dataTypes.ExtendedHash(), hasmore;
		hasmore = istream.readLine(line);
		
		if(BookmarkTools.utils.ObjectUtils.isNotUndefined(delim)){
			var tmp;
			while(hasmore){
				tmp = line.value.split(delim);
				lines.put(tmp[0], tmp[1]); 
				hasmore = istream.readLine(line);
			}
		}else {
			var count = 0;
			while(hasmore){
				lines.put(line.value, count); 
				hasmore = istream.readLine(line);
				count++;
			}
		}

		istream.close();

		return lines;
	},
	
	/**
		Reads an file and returned the content as string.
		@param {nsIFile} file
		@return file content as string
	*/
	readFile : function(file){
		var data = "";
		var fstream = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(Ci.nsIFileInputStream);
		var sstream = Cc["@mozilla.org/scriptableinputstream;1"].createInstance(Ci.nsIScriptableInputStream);
		fstream.init(file, -1, 0, 0);
		sstream.init(fstream); 

		var str = sstream.read(4096);
		while (str.length > 0) {
		  data += str;
		  str = sstream.read(4096);
		}

		sstream.close();
		fstream.close();
		
		return data;
	},
	
	/**
		Creates a new empty file. The path is given as slash separated string.
		@param {string} path path where the file is created
		@return file instance
	*/
	newFile : function(path){
		var subFolders = path.split('/');
		var file = Cc["@mozilla.org/file/directory_service;1"]
		.getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
		
		for(var i = 0, l = subFolders.length ;i < l; i++)
			file.append(subFolders[i]);
			
		if(!file.exists())
			file.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, 0666);
			
		return file;
	},
	
	/**
		Lists all files which are available under the specified path.
		@param {string} path path to list the files
		@return null or a list of files
	*/
	listFiles : function(path){
		var subFolders = path.split('/');
		var dir = Cc["@mozilla.org/file/directory_service;1"]
		.getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
		
		for(var i = 0, l = subFolders.length ;i < l; i++)
			dir.append(subFolders[i]);
			
		if(dir.exists() && dir.isDirectory())
			return dir.directoryEntries;
			
		return null;
	}

};
